@@ -52,6 +52,7 @@ import org.apache.naming.resources.Resource;
 import org.apache.tomcat.util.buf.CharChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.http.mapper.MappingData;
+import org.apache.catalina.Globals;
 
 
 /**
@@ -61,7 +62,7 @@ import org.apache.tomcat.util.http.mapper.MappingData;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 606606 $ $Date: 2007-12-23 21:09:23 +0100 (dim., 23 déc. 2007) $
+ * @version $Revision: 673839 $ $Date: 2008-07-04 00:07:04 +0200 (Fri, 04 Jul 2008) $
  */
 
 public class ApplicationContext
@@ -368,10 +369,21 @@ public class ApplicationContext
             throw new IllegalArgumentException
                 (sm.getString
                  ("applicationContext.requestDispatcher.iae", path));
+
+        // Get query string
+        String queryString = null;
+        int pos = path.indexOf('?');
+        if (pos >= 0) {
+            queryString = path.substring(pos + 1);
+            path = path.substring(0, pos);
+        }
+
         path = normalize(path);
         if (path == null)
             return (null);
 
+        pos = path.length(); 
+
         // Use the thread local URI and mapping data
         DispatchData dd = dispatchData.get();
         if (dd == null) {
@@ -382,15 +394,6 @@ public class ApplicationContext
         MessageBytes uriMB = dd.uriMB;
         uriMB.recycle();
 
-        // Get query string
-        String queryString = null;
-        int pos = path.indexOf('?');
-        if (pos >= 0) {
-            queryString = path.substring(pos + 1);
-        } else {
-            pos = path.length();
-        }
- 
         // Use the thread local mapping data
         MappingData mappingData = dd.mappingData;
 
@@ -453,9 +456,12 @@ public class ApplicationContext
     public URL getResource(String path)
         throws MalformedURLException {
 
-        if (path == null || !path.startsWith("/")) {
+        if (path == null)
             throw new MalformedURLException(sm.getString("applicationContext.requestDispatcher.iae", path));
-        }
+
+        if (!path.startsWith("/") && Globals.STRICT_SERVLET_COMPLIANCE)
+            throw new MalformedURLException(sm.getString("applicationContext.requestDispatcher.iae", path));
+
         
         path = normalize(path);
         if (path == null)
@@ -507,9 +513,12 @@ public class ApplicationContext
     public InputStream getResourceAsStream(String path) {
 
         path = normalize(path);
-        if (path == null || !path.startsWith("/"))
+        if (path == null)
             return (null);
 
+        if (!path.startsWith("/") && Globals.STRICT_SERVLET_COMPLIANCE)
+            return null;
+        
         DirContext resources = context.getResources();
         if (resources != null) {
             try {
@@ -957,7 +966,7 @@ public class ApplicationContext
      * Internal class used as thread-local storage when doing path
      * mapping during dispatch.
      */
-    private final class DispatchData {
+    private static final class DispatchData {
 
         public MessageBytes uriMB;
         public MappingData mappingData;
