@@ -64,7 +64,7 @@ import org.xml.sax.SAXParseException;
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Revision: 556489 $ $Date: 2007-07-16 04:01:01 +0200 (lun., 16 juil. 2007) $
+ * @version $Revision: 656829 $ $Date: 2008-05-15 22:51:54 +0200 (Thu, 15 May 2008) $
  */
 
 public class ContextConfig
@@ -339,10 +339,6 @@ public class ContextConfig
         
         long t1=System.currentTimeMillis();
 
-        if (webDigester == null){
-            webDigester = createWebDigester();
-        }
-        
         URL url=null;
         // Process the application web.xml file
         synchronized (webDigester) {
@@ -624,10 +620,6 @@ public class ContextConfig
                       + " " + defaultWebXml + " " + file , e);
         }
 
-        if (webDigester == null){
-            webDigester = createWebDigester();
-        }
-        
         if (stream != null) {
             processDefaultWebConfig(webDigester, stream, source);
             webRuleSet.recycle();
@@ -786,9 +778,6 @@ public class ContextConfig
         
         if (source == null)
             return;
-        if (contextDigester == null){
-            contextDigester = createContextDigester();
-        }
         synchronized (contextDigester) {
             try {
                 source.setByteStream(stream);
@@ -884,9 +873,13 @@ public class ContextConfig
         String contextPath = context.getPath();
         if (contextPath.equals("")) {
             contextPath = "ROOT";
+        } else {
+            if (contextPath.lastIndexOf('/') > 0) {
+                contextPath = "/" + contextPath.substring(1).replace('/','#');
+            }
         }
         if (docBase.toLowerCase().endsWith(".war") && !file.isDirectory() && unpackWARs) {
-            URL war = new URL("jar:" + (new File(docBase)).toURL() + "!/");
+            URL war = new URL("jar:" + (new File(docBase)).toURI().toURL() + "!/");
             docBase = ExpandWar.expand(host, war, contextPath);
             file = new File(docBase);
             docBase = file.getCanonicalPath();
@@ -899,7 +892,8 @@ public class ContextConfig
                 File warFile = new File(docBase + ".war");
                 if (warFile.exists()) {
                     if (unpackWARs) {
-                        URL war = new URL("jar:" + warFile.toURL() + "!/");
+                        URL war =
+                            new URL("jar:" + warFile.toURI().toURL() + "!/");
                         docBase = ExpandWar.expand(host, war, contextPath);
                         file = new File(docBase);
                         docBase = file.getCanonicalPath();
@@ -997,6 +991,16 @@ public class ContextConfig
     protected void init() {
         // Called from StandardContext.init()
 
+        if (webDigester == null){
+            webDigester = createWebDigester();
+            webDigester.getParser();
+        }
+        
+        if (contextDigester == null){
+            contextDigester = createContextDigester();
+            contextDigester.getParser();
+        }
+
         if (log.isDebugEnabled())
             log.debug(sm.getString("contextConfig.init"));
         context.setConfigured(false);
