@@ -43,7 +43,7 @@ import org.xml.sax.Attributes;
  * deployment descriptor (<code>/WEB-INF/web.xml</code>) resource.</p>
  *
  * @author Craig R. McClanahan
- * @version $Revision: 550263 $ $Date: 2007-06-24 19:28:45 +0200 (dim., 24 juin 2007) $
+ * @version $Revision: 628307 $ $Date: 2008-02-16 15:40:24 +0100 (Sat, 16 Feb 2008) $
  */
 
 public class WebRuleSet extends RuleSetBase {
@@ -256,9 +256,6 @@ public class WebRuleSet extends RuleSetBase {
         digester.addCallMethod(prefix + "web-app/jsp-config/jsp-property-group/url-pattern",
                                "addJspMapping", 0);
 
-        digester.addCallMethod(prefix + "web-app/listener/listener-class",
-                               "addApplicationListener", 0);
-        
         digester.addRule(prefix + "web-app/login-config",
                          loginConfig);
 
