@@ -125,7 +125,7 @@ import org.apache.juli.logging.LogFactory;
  * @author Tim Funk
  * @author Peter Rossbach
  * 
- * @version $Revision: 575411 $ $Date: 2007-09-13 21:31:16 +0200 (jeu., 13 sept. 2007) $
+ * @version $Revision: 633907 $ $Date: 2008-03-05 17:18:09 +0100 (Wed, 05 Mar 2008) $
  */
 
 public class ExtendedAccessLogValve
@@ -663,7 +663,7 @@ public class ExtendedAccessLogValve
                         public void addElement(StringBuffer buf, Date date,
                                 Request request, Response response, long time) {
                             String query = request.getQueryString();
-                            if (query != null) {
+                            if (query == null) {
                                 buf.append(request.getRequestURI());
                             } else {
                                 buf.append(request.getRequestURI());
